package javawrapper;

import java.util.Vector;

/**
* This class packages vector data with other pieces of information important
* to the model.  This allows a vector of data to self-describe to error
* messages and display windows, and to identify the data that belongs to it
* in XML files being read and written.
* <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
* <p>Company: Institute of Ecosystem Studies</p>
* @author Lora E. Murphy
* @version 1.0
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)
* <br>October 7, 2004:  Added the m_bMustApplyToAllSpecies member (LEM)
* <br>January 22, 2005: Added the m_bIsForSpecies member (LEM)
* <br>February 4, 2008: Updated to conform to Java 6 compliance (LEM)
*/

public class ModelVector
    extends ModelData {

  /**The data values*/
  private Vector<Object> p_oData;
  /**This is the XML tag of the individual values*/
  private String m_sXMLChildTag;
  /**Data type desired by this ModelVector*/
  private int m_iDataType;
  /**Whether this is a set of species-specific values - not all vectors are*/
  private boolean m_bIsForSpecies = true;
  /**Whether or not this automatically applies to all species.  If false, then
   * which species values are contained in this vector is controlled by such
   * factors as what behaviors claim it and what species they apply to.*/
  private boolean m_bMustApplyToAllSpecies = false;

  /**This vector desires data of type Integer*/
  public final static int INTEGER = 0;

  /**This vector desires data of type Float*/
  public final static int FLOAT = 1;

  /**This vector desires data of type ModelEnum*/
  public final static int MODEL_ENUM = 2;

  /**This vector desires data of type String*/
  public final static int STRING = 3;

  /**
   * Required overridden method.
   * @return null - this cannot be applied.
   */
  public String toString() {
    return null;
  }

  /**
   * Constructor
   * @param sDescriptor A descriptive name string for this value.  This will
   * appear in the data window and in error messages.
   * @param sXMLTag The XML tag of the whole vector
   * @param sXMLChildTag The XML tag of individual values within the vector
   * @param iSize Initial size of the vector.  0 is always safe (since vectors
   * grow to accommodate whatever data is placed in them).
   * @param iDesiredDataType Desired data type of this vector.
   */
  ModelVector(String sDescriptor, String sXMLTag, String sXMLChildTag,
              int iSize,
              int iDesiredDataType) {
    this(sDescriptor, sXMLTag, sXMLChildTag, iSize, iDesiredDataType, false);
  }

  /**
   * Constructor
   * @param sDescriptor A descriptive name string for this value.  This will
   * appear in the data window and in error messages.
   * @param sXMLTag The XML tag of the whole vector
   * @param sXMLChildTag The XML tag of individual values within the vector
   * @param iSize Initial size of the vector.  0 is always safe (since vectors
   * grow to accommodate whatever data is placed in them).
   * @param iDesiredDataType Desired data type of this vector.
   * @param bMustApplyToAllSpecies Whether or not this vector automatically
   * applies to all species, overriding any other method of determining
   * which species it applies to.
   */
  ModelVector(String sDescriptor, String sXMLTag, String sXMLChildTag,
              int iSize, int iDesiredDataType,
              boolean bMustApplyToAllSpecies) {
    super(sDescriptor, sXMLTag);
    p_oData = new Vector<Object>(iSize);
    m_sXMLChildTag = sXMLChildTag;
    m_iDataType = iDesiredDataType;
    m_bMustApplyToAllSpecies = bMustApplyToAllSpecies;
  }

  /**
   * Gets the XML tag used to identify each piece of data in the vector.
   * @return The XML tag for this vector's data.
   */
  public String GetChildXMLTag() {
    return m_sXMLChildTag;
  }

  /**
   * Gets whether this vector must apply to all tree species, regardless of
   * other factors controlling species.
   * @return boolean Whether this vector must apply to all species.
   */
  public boolean GetMustApplyToAllSpecies() {
    return m_bMustApplyToAllSpecies;
  }

  /**
   * Gets whether this vector is for species-specific values.
   * @return boolean Whether this vector is for species-specific values.
   */
  public boolean GetIsSpeciesSpecific() {
    return m_bIsForSpecies;
  }

  /**
   * Sets whether this vector is for species-specific values.
   * @param bIsSpeciesSpecific boolean Whether this vector is for species-
   * specific values.
   */
  public void SetIsSpeciesSpecific(boolean bIsSpeciesSpecific) {
    m_bIsForSpecies = bIsSpeciesSpecific;
  }

  /**
   * Gets the Vector values.
   * @return The Vector values.
   */
  public Vector<Object> GetValue() {
    return p_oData;
  }

  /**
   * Gets the data type desired for this ModelVector.
   * @return The desired data type.
   */
  public int GetDataType() {
    return m_iDataType;
  }
}
