package javawrapper;

import java.util.Vector;

/**
* Plot object.
* Copyright: Copyright (c) Charles D. Canham 2003
* Company: Institute of Ecosystem Studies
* @author Lora E. Murphy
* @version 1.0
*
* <br>Edit history:
* <br>------------------
* <br>April 28, 2004: Submitted in beta version (LEM)
* <br>March 1, 2007: Checked for plot size changes so we can update
* grids (LEM)
* <br>February 4, 2008: Updated to conform to Java 6 compliance (LEM)
*/

public class Plot
    extends WorkerBase {

  /** Length of cells, in meters. */
  public static final float CELL_LENGTH = (float)8.0;

  protected ModelString
      /**Plot title or comment*/
      m_sPlotTitle = new ModelString("", "Plot title", "plot_title");

  /**Length of plot in X direction.  X is EW.*/
  protected ModelFloat m_fPlotLenX = new ModelFloat(0,
      "Plot Length in the X (E-W) Direction, in meters", "plot_lenX");

  /**Length of plot in Y direction.  Y is NS.*/
  protected ModelFloat m_fPlotLenY = new ModelFloat(0,
      "Plot Length in the Y (N-S) Direction, in meters", "plot_lenY");

  /**Latitude of plot in decimal degrees*/
  protected ModelFloat m_fLatitude = new ModelFloat(0,
      "Plot Latitude, in decimal degrees", "plot_latitude");

  /**Number of years per timestep - default to 5, the old value*/
  protected ModelFloat m_fNumberOfYearsPerTimestep = new ModelFloat(5,
      "Number of years per timestep", "yearsPerTimestep");

  /**Number of timesteps for the run*/
  protected ModelInt m_iNumTimesteps = new ModelInt(0, "Number of Timesteps",
      "timesteps");

  /**Run's random seed*/
  protected ModelInt m_iRandomSeed = new ModelInt(0, "Random Seed",
                                                  "randomSeed");

  /**Current timestep, if a detailed output timestep file has
   been fed in.  Not required.*/
  protected ModelInt m_iCurrentTimestep = new ModelInt(0, "Current Timestep",
      "rt_timestep");

  /**
   * Default constructor initializes all values to the equivalent of 0.
   * @param oManager GUIManager object.
   */
  public Plot(GUIManager oManager) {
    super(oManager, "Plot", "plot");

    m_sHelpID = "data.plot";

    mp_oAllData.add(m_iNumTimesteps);
    mp_oAllData.add(m_iCurrentTimestep);
    mp_oAllData.add(m_iRandomSeed);
    mp_oAllData.add(m_fNumberOfYearsPerTimestep);
    mp_oAllData.add(m_fPlotLenX);
    mp_oAllData.add(m_fPlotLenY);
    mp_oAllData.add(m_fLatitude);
    mp_oAllData.add(m_sPlotTitle);
}

  /**Does nothing
   * @param oPop Ignored*/
  public void DoSetup(TreePopulation oPop){;}

  /**
   * Sets the number of years per timestep.
   * @param fYears Number of years per timestep.
   * @throws ModelException if the number of years per timestep is not greater
   * than 0.
   */
  public void SetYearsPerTimestep(float fYears) throws ModelException {
    ModelFloat fTest = new ModelFloat(fYears,
                                      m_fNumberOfYearsPerTimestep.GetDescriptor(),
                                      "");
    MakeSureGreaterThan(fTest, 0);
    m_fNumberOfYearsPerTimestep.SetValue(fYears);
  }

  /**
   * Gets the number of years per timestep.
   * @return Number of years per timestep.
   */
  public float GetNumberOfYearsPerTimestep() {
    return m_fNumberOfYearsPerTimestep.GetValue();
  }

  /**
   * Sets the plot title.
   * @param sTitle Title of the plot.
   */
  public void SetPlotTitle(String sTitle) {
    m_sPlotTitle.SetValue(sTitle);
  }

  /**
   * Sets the number of timesteps for a run.
   * @param i The number of timesteps.
   * @throws ModelException when the number of timesteps is less than or equal
   * to zero.
   */
  public void SetNumberOfTimesteps(int i) throws ModelException {
    ModelInt iTest = new ModelInt(i, m_iNumTimesteps.GetDescriptor(), "");
    MakeSureGreaterThan(iTest, 0);
    m_iNumTimesteps.SetValue(i);
  }

  /**
   * Gets the number of timesteps for a run.
   * @return The number of timesteps.
   */
  public int GetNumberOfTimesteps() {
    return m_iNumTimesteps.GetValue();
  }

  /**
   * Sets the random seed for the run.  A value of 0 means a new random seed each time a parameter file is run.
   * @param i The random seed value.
   */
  public void SetRandomSeed(int i) {
    m_iRandomSeed.SetValue(i);
  }

  /**
   * Sets the plot length in the X (East-West) direction.
   * @param f Length of the plot in the X direction.
   * @throws ModelException if the passed length is less than or equal to 0.
   */
  public void SetPlotXLength(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fPlotLenX.GetDescriptor(), "");
    MakeSureGreaterThan(fTest, 0);
    m_fPlotLenX.SetValue(f);
  }

  /**
   * Gets the current plot X (East-West) length.
   * @return Current plot X length.
   */
  public float GetPlotXLength() {
    return m_fPlotLenX.GetValue();
  }

  /**
   * Sets the plot length in the Y (North-South) direction.
   * @param f Length of the plot in the Y direction.
   * @throws ModelException if the passed length is less than or equal to 0.
   */
  public void SetPlotYLength(float f) throws ModelException {
    ModelFloat fTest = new ModelFloat(f, m_fPlotLenY.GetDescriptor(), "");
    MakeSureGreaterThan(fTest, 0);
    m_fPlotLenY.SetValue(f);
  }

  /**
   * Gets the current plot Y (North-South) length.
   * @return Current plot Y length.
   */
  public float GetPlotYLength() {
    return m_fPlotLenY.GetValue();
  }

  /**
   * Sets the latitude of the plot.
   * @param f Latitude in decimal degrees.
   * @throws ModelException if latitude is not between -90 and 90.
   */
  public void SetLatitude(float f) throws ModelException {
    if (f > 90 || f < -90) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                               m_fLatitude.GetDescriptor() +
                               " must be between -90 and 90."));
    }
    m_fLatitude.SetValue(f);
  }

  /**
   * Validates the data prior to running the model.
   * @throws ModelException if any of the following are not greater than zero:
   * <ul>
   * <li> m_fPlotLenX
   * <li> m_fPlotLenY
   * <li> m_iNumTimesteps
   * <li> m_fNumberOfYearsPerTimestep
   * </ul>
   * or if m_fLatitude is not between -90 and +90.
   * @param oPop Not used.
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
  	MakeSureGreaterThan(m_fPlotLenX, 0);
  	MakeSureGreaterThan(m_fPlotLenY, 0);
  	MakeSureGreaterThan(m_iNumTimesteps, 0);
  	MakeSureGreaterThan(m_fNumberOfYearsPerTimestep, 0);
  	MakeSureIsBounded(m_fLatitude, -90, 90);
  }

  /**
   * Checks for changes to plot size, and tells the GUIManager if there are any.
   * @param p_oData Vector Vector of data
   * @param oPop TreePopulation Tree population object
   * @throws ModelException won't.
   */
  public void ReadDataFromDisplay(Vector<TableData> p_oData, TreePopulation oPop) throws
      ModelException {
    //Capture the existing plot size
    float fX = m_fPlotLenX.GetValue(), fY = m_fPlotLenY.GetValue();
    super.ReadDataFromDisplay(p_oData, oPop);
    if (java.lang.Math.abs(fX - m_fPlotLenX.GetValue()) > 0.001 ||
        java.lang.Math.abs(fY - m_fPlotLenY.GetValue()) > 0.001) {
      m_oManager.ChangeOfPlotResolution(fX, fY, m_fPlotLenX.GetValue(), m_fPlotLenY.GetValue());
    }
  }
}
