package javawrapper;

import junit.framework.*;
import java.io.FileWriter;
import java.io.File;

/**
 * Tests the WorkerBase class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class TestBehaviorTypeBase extends TestCase {

  /**
   * Tests change of species
   */
  public void testChangeOfSpecies() {
    String sFileName = null;
    try {
      GUIManager oManager = new GUIManager(null);
      sFileName = WriteXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      LightBehaviors oLight = oManager.GetLightBehaviors();
      Behavior oBeh = oLight.GetBehaviorByXMLTag("quadratlight");
      assertEquals(5, oBeh.GetNumberOfCombos());

      String[] sNewSpecies = new String[] {
            "Test 1",
            "TSCA",
            "QURU",
            "PRSE",
            "PIST",
            "Test 2",
            "FRAM",
            "BEAL",
            "ACSA",
            "ACRU"
      };
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByXMLTag("quadratlight");
      assertEquals(4, oBeh.GetNumberOfCombos());
      SpeciesTypeCombo oCombo = oBeh.GetSpeciesTypeCombo(0);
      assertEquals(9, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(1);
      assertEquals(8, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(2);
      assertEquals(7, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(3);
      assertEquals(2, oCombo.GetSpecies());

      ///////////////////////////////////////
      // Test that when all species are removed for
      // a behavior it is un-enabled
      ///////////////////////////////////////
      oManager.InputXMLParameterFile(sFileName);
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByXMLTag("glilight");
      assertEquals(1, oBeh.GetNumberOfCombos());
      assertTrue(oBeh.m_bIsEnabled);
      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByXMLTag("storm");
      assertTrue(oBeh.m_bIsEnabled);

      sNewSpecies = new String[] {
            "Test 1",
            "TSCA",
            "QURU",
            "PRSE",
            "Test 2",
            "FRAM",
            "BEAL",
            "ACSA",
            "ACRU"
      };
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByXMLTag("glilight");
      assertEquals(0, oBeh.GetNumberOfCombos());
      assertFalse(oBeh.m_bIsEnabled);
      //Make sure a behavior that does not apply to trees is unaffected
      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByXMLTag("storm");
      assertTrue(oBeh.m_bIsEnabled);

    } catch (ModelException oErr) {
      fail("ChangeOfSpecies testing failed with message " + oErr.getMessage());
    } finally {
      new File(sFileName).delete();
    }
    System.out.println("ChangeOfSpecies testing succeeded.");
  }

  /**
   * Tests copying of species
   */
  public void testCopySpecies() {
    String sFileName = null;
    try {
      GUIManager oManager = new GUIManager(null);
      sFileName = WriteXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      LightBehaviors oLight = oManager.GetLightBehaviors();
      Behavior oBeh = oLight.GetBehaviorByXMLTag("quadratlight");
      assertEquals(5, oBeh.GetNumberOfCombos());

      //Copy ACSA to BEAL
      oManager.GetTreePopulation().DoCopySpecies("ACSA", "BEAL");
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByXMLTag("quadratlight");
      assertEquals(5, oBeh.GetNumberOfCombos());
      SpeciesTypeCombo oCombo = oBeh.GetSpeciesTypeCombo(0);
      assertEquals(0, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(1);
      assertEquals(1, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(2);
      assertEquals(3, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(3);
      assertEquals(8, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(4);
      assertEquals(2, oCombo.GetSpecies());

      oBeh = oLight.GetBehaviorByXMLTag("glilight");
      assertEquals(1, oBeh.GetNumberOfCombos());
      oCombo = oBeh.GetSpeciesTypeCombo(0);
      assertEquals(6, oCombo.GetSpecies());

      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByXMLTag("storm");
      assertTrue(oBeh.m_bIsEnabled);

      ///////////////////////////////////////
      // Test that when all species are removed for
      // a behavior it is un-enabled
      // Copy BEAL to PIST
      ///////////////////////////////////////
      oManager.GetTreePopulation().DoCopySpecies("BEAL", "PIST");
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByXMLTag("quadratlight");
      assertEquals(6, oBeh.GetNumberOfCombos());
      oCombo = oBeh.GetSpeciesTypeCombo(0);
      assertEquals(0, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(1);
      assertEquals(1, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(2);
      assertEquals(3, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(3);
      assertEquals(8, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(4);
      assertEquals(2, oCombo.GetSpecies());
      oCombo = oBeh.GetSpeciesTypeCombo(5);
      assertEquals(6, oCombo.GetSpecies());

      oBeh = oLight.GetBehaviorByXMLTag("glilight");
      assertEquals(0, oBeh.GetNumberOfCombos());
      assertFalse(oBeh.m_bIsEnabled);

      oBeh = oManager.GetDisturbanceBehaviors().GetBehaviorByXMLTag("storm");
      assertTrue(oBeh.m_bIsEnabled);

    } catch (ModelException oErr) {
      fail("ChangeOfSpecies testing failed with message " + oErr.getMessage());
    } finally {
      new File(sFileName).delete();
    }
    System.out.println("ChangeOfSpecies testing succeeded.");
  }

  /**
   * This writes an XML file to test species changing.  This file contains
   * light information.
   * @return Name of file.
   * @throws ModelException if there is an IO problem.
   */
  protected String WriteXMLFile1() throws ModelException {
    try {
      String sFileName = "\\loratestxml1.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<paramFile fileCode=\"06050101\">");
      oOut.write("<plot>");
      oOut.write("<timesteps>400</timesteps>");
      oOut.write("<randomSeed>0</randomSeed>");
      oOut.write("<yearsPerTimestep>5.0</yearsPerTimestep>");
      oOut.write("<plot_lenX>500.0</plot_lenX>");
      oOut.write("<plot_lenY>500.0</plot_lenY>");
      oOut.write("<plot_latitude>41.92</plot_latitude>");
      oOut.write("</plot>");
      oOut.write("<trees>");
      oOut.write("<tr_speciesList>");
      oOut.write("<tr_species speciesName=\"ACRU\"/>");
      oOut.write("<tr_species speciesName=\"ACSA\"/>");
      oOut.write("<tr_species speciesName=\"BEAL\"/>");
      oOut.write("<tr_species speciesName=\"FAGR\"/>");
      oOut.write("<tr_species speciesName=\"TSCA\"/>");
      oOut.write("<tr_species speciesName=\"FRAM\"/>");
      oOut.write("<tr_species speciesName=\"PIST\"/>");
      oOut.write("<tr_species speciesName=\"PRSE\"/>");
      oOut.write("<tr_species speciesName=\"QURU\"/>");
      oOut.write("</tr_speciesList>");
      oOut.write("<tr_seedDiam10Cm>0.2</tr_seedDiam10Cm>");
      oOut.write("</trees>");
      oOut.write("<behaviorList>");
      oOut.write("<behavior>");
      oOut.write("<behaviorName>storm</behaviorName>");
      oOut.write("<version>1</version>");
      oOut.write("</behavior>");
      oOut.write("<behavior>");
      oOut.write("<behaviorName>quadratlight</behaviorName>");
      oOut.write("<version>1.0</version>");
      oOut.write("<applyTo species=\"ACRU\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"ACSA\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"BEAL\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"FAGR\" type=\"Seedling\"/>");
      oOut.write("<applyTo species=\"QURU\" type=\"Seedling\"/>");
      oOut.write("</behavior>");
      oOut.write("<behavior>");
      oOut.write("<behaviorName>glilight</behaviorName>");
      oOut.write("<version>1</version>");
      oOut.write("<applyTo species=\"PIST\" type=\"Adult\"/>");
      oOut.write("</behavior>");
      oOut.write("</behaviorList>");
      oOut.write("<gliLight>");
      oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
      oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
      oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
      oOut.write("<li_julianDayGrowthStarts>120</li_julianDayGrowthStarts>");
      oOut.write("<li_julianDayGrowthEnds>270</li_julianDayGrowthEnds>");
      oOut.write("<li_minSunAngle>0.779</li_minSunAngle>");
      oOut.write("<li_numAltGrids>12</li_numAltGrids>");
      oOut.write("<li_numAziGrids>18</li_numAziGrids>");
      oOut.write("<li_lightExtinctionCoefficient>");
      oOut.write("<li_lecVal species=\"ACRU\">0.1</li_lecVal>");
      oOut.write("<li_lecVal species=\"ACSA\">0.2</li_lecVal>");
      oOut.write("<li_lecVal species=\"BEAL\">0.3</li_lecVal>");
      oOut.write("<li_lecVal species=\"FAGR\">0.4</li_lecVal>");
      oOut.write("<li_lecVal species=\"TSCA\">0.5</li_lecVal>");
      oOut.write("<li_lecVal species=\"FRAM\">0.6</li_lecVal>");
      oOut.write("<li_lecVal species=\"PIST\">0.7</li_lecVal>");
      oOut.write("<li_lecVal species=\"PRSE\">0.8</li_lecVal>");
      oOut.write("<li_lecVal species=\"QURU\">0.9</li_lecVal>");
      oOut.write("</li_lightExtinctionCoefficient>");
      oOut.write("<li_snagAgeClass1>0</li_snagAgeClass1>");
      oOut.write("<li_snagAgeClass2>0</li_snagAgeClass2>");
      oOut.write("</gliLight>");
      oOut.write("</paramFile>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }
}
